import 'package:fast_immutable_collections/fast_immutable_collections.dart';
import 'package:flutter/material.dart';

import 'package:loading_more_list_library_fast/loading_more_list_library_fast.dart';
import 'package:loading_more_list_library_fast/model/status.dart';
import 'package:provider/provider.dart';

import '../loading_more_list_fast.dart';

//loading more for listview and gridview
class LoadingMoreList<T> extends StatelessWidget {
  const LoadingMoreList(this.listConfig, {Key? key, this.onScrollNotification})
      : super(key: key);
  final ListConfig<T> listConfig;

  /// Called when a ScrollNotification of the appropriate type arrives at this
  /// location in the tree.
  final NotificationListenerCallback<ScrollNotification>? onScrollNotification;

  @override
  Widget build(BuildContext context) {
    final LoadingMoreBase<T> sourceList = listConfig.sourceList;
    final bool useProviderValue = listConfig.useProviderValue;

    final Selector<LoadingMoreBase<T>, IList<T>> child =
        Selector<LoadingMoreBase<T>, IList<T>>(
            builder: (BuildContext context, IList<T> value, Widget? child) {
              return Selector<LoadingMoreBase<T>, IndicatorStatusModel>(
                  builder: (BuildContext context, IndicatorStatusModel value,
                          Widget? child) =>
                      listConfig.buildContent(context, sourceList),
                  selector: (BuildContext p0, LoadingMoreBase<T> p1) =>
                      p1.indicatorStatus);
            },
            selector: (BuildContext p0, LoadingMoreBase<T> p1) => p1.array);
    final ChangeNotifierProvider<LoadingMoreBase<T>> widget = useProviderValue
        ? ChangeNotifierProvider<LoadingMoreBase<T>>.value(
            value: sourceList,
            child: child,
          )
        : ChangeNotifierProvider<LoadingMoreBase<T>>(
            create: (BuildContext context) => sourceList, child: child);

    return NotificationListener<ScrollNotification>(
      onNotification: _handleScrollNotification,
      child: GlowNotificationWidget(
        widget,
        showGlowLeading: listConfig.showGlowLeading,
        showGlowTrailing: listConfig.showGlowTrailing,
      ),
    );
  }

  bool _handleScrollNotification(ScrollNotification notification) {
    if (onScrollNotification != null) {
      onScrollNotification!(notification);
    }

    if (notification.depth != 0) {
      return false;
    }

    //reach the pixels to loading more
    if (notification.metrics.pixels >= notification.metrics.maxScrollExtent) {
      if (listConfig.hasMore && !listConfig.hasError && !listConfig.isLoading) {
        if (listConfig.sourceList.isEmpty) {
          if (listConfig.autoRefresh) {
            listConfig.sourceList.refresh();
          }
        } else if (listConfig.autoLoadMore) {
          listConfig.sourceList.loadMore();
        }
      }
    }
    return false;
  }
}
